---------------------------------------------------------------------------------------------------------------

local addonName = ...
local SOUND_LIST_VERSION = GetAddOnMetadata(addonName, "Version")

---------------------------------------------------------------------------------------------------------------

local _G = _G
local maxLines = nil;
local maxLinesToShow = 20;
local willPlay, soundHandle, selectedButtonID
local SOUNDLIST_KEYWORDS = { "All Sounds", "Custom List", "Other Sounds", "", "Ambience", "Character", "Creature", "Doodad", "Emitters", "Interface", "Item", "Music", "Spells" }
local SOUND_LIST = {}

---------------------------------------------------------------------------------------------------------------

local function SoundList_StopSound()
	if soundHandle then 
		StopSound(soundHandle)
		soundHandle = nil
	end
	selectedButtonID = nil
end

---------------------------------------------------------------------------------------------------------------

local function SoundList_RefreshList()
--print("SoundList_RefreshList()")
	maxLines = #SOUND_LIST
	local title = SOUNDLIST_KEYWORDS[SoundListOptionsFrame.selectedID]
SoundList_StopSound()
	SoundListFrameScrollBar:Show()
	FauxScrollFrame_OnVerticalScroll(SoundListFrameScrollBar, 0, 10, SoundListFrameScrollBar_Update)
	SoundListFrameScrollBar_Update()
	SoundListFrame.title:SetText(format("%s v%s - %s (total: %i)", addonName, SOUND_LIST_VERSION, title, #SOUND_LIST))
end

---------------------------------------------------------------------------------------------------------------

 local function SoundList_CreateButtons()
--print("SoundList_CreateButtons()")
	local btn = nil;
	for i=2, maxLinesToShow do
		btn = CreateFrame("Button", "slb" .. i, SoundListFrame, "slbt")
		btn:SetPoint("TOPLEFT", "slb" .. i-1, "BOTTOMLEFT", 0, -1)
	end
--	SoundList_RefreshList()
end

---------------------------------------------------------------------------------------------------------------

local function shallowcopy(orig)
	local orig_type = type(orig)
	local copy
	if orig_type == 'table' then
		copy = {}
		for orig_key, orig_value in pairs(orig) do
			copy[orig_key] = orig_value
		end
	else -- number, string, boolean, etc
		copy = orig
	end
	return copy
end

---------------------------------------------------------------------------------------------------------------

local function SoundListOptions_SelectButton(btn_id)
	local btn
	if SoundListOptionsFrame.selectedID then
		btn = _G["sl_op_btn" .. SoundListOptionsFrame.selectedID]
		btn:GetNormalTexture():SetGradientAlpha("VERTICAL", 0.1, 0.1, 0.1, 0, 0.25, 0.25, 0.25, 0.8)
	end
	btn = _G["sl_op_btn" .. btn_id]
	btn:GetNormalTexture():SetGradientAlpha("VERTICAL", 1, 0.8, 0, 0.7, 0, 0.6, 0, 0.5)
	SoundListOptionsFrame.selectedID = btn_id
end

---------------------------------------------------------------------------------------------------------------

local function SoundList_ToggleSoundsList(btn_id)

--local startTime = debugprofilestop()

	local str, str_upper, title, keyword

	keyword = "/" .. string.upper(SOUNDLIST_KEYWORDS[btn_id]) .. "/"

	wipe(SOUND_LIST)

	if btn_id == 1 then -- All Sounds
		SOUND_LIST = shallowcopy(SOUND_LIST_SOUNDS)

	elseif btn_id == 2 then -- Custom List
		SOUND_LIST = shallowcopy(SOUND_LIST_SOUNDS_CUSTOM)

	elseif btn_id == 3 then -- Other Sounds
		local match
		for i=1, #SOUND_LIST_SOUNDS do
			match = false
			str = SOUND_LIST_SOUNDS[i]
			str_upper = string.upper(str)
			for j=5, #SOUNDLIST_KEYWORDS do
				keyword = "/" .. string.upper(SOUNDLIST_KEYWORDS[j]) .. "/"
				if string.find(str_upper, keyword) then
					match = true
					break
				end
			end
			if not match then
				table.insert(SOUND_LIST, str)
			end
		end

	-- ******************** SAME AS ABOVE CODE BUT A BIT SLOWER
	--		local match
	--		for i=1, #SOUND_LIST_SOUNDS do
	--			match = false
	--			str = SOUND_LIST_SOUNDS[i]
	--			str_upper = string.upper(str)
	--			for j=5, #SOUNDLIST_KEYWORDS do
	--				keyword = "/" .. string.upper(SOUNDLIST_KEYWORDS[j]) .. "/"
	--				match = match or string.find(str_upper, keyword)
	--				if match then break end
	--			end
	--			if not match then
	--				table.insert(SOUND_LIST, str)
	--			end
	--		end
	-- ******************** SAME AS ABOVE CODE BUT A BIT SLOWER



	else
		for i=1, #SOUND_LIST_SOUNDS do
			str = SOUND_LIST_SOUNDS[i]
			str_upper = string.upper(str)
			if string.find(str_upper, keyword) then
				table.insert(SOUND_LIST, str)
			end
		end
	end

	-- Select the button
	SoundListOptions_SelectButton(btn_id)

	if SoundListOptionsFrameSearch:GetText() ~= "" then
		SoundListOptions_Search(SoundListOptionsFrameSearch:GetText())
	end
	SoundList_RefreshList()

--/run msg( 1138+820+6047+49263+4091+261+2320+3250  )
--msg(btn_id, #SOUND_LIST, format("myFunction executed in %f ms", debugprofilestop()-startTime))
end

---------------------------------------------------------------------------------------------------------------

local function SoundListOptions_ShowOptions()
	if SoundListOptionsFrame:IsShown() then
--        UIFrameFadeOut(SoundListOptionsFrame, 0.1)
		SoundListOptionsFrame:Hide()
	else
		UIFrameFadeIn(SoundListOptionsFrame, 0.5)
		SoundListOptionsFrame:Show()
	end
end

---------------------------------------------------------------------------------------------------------------

function SoundList_OnLoad()
	DEFAULT_CHAT_FRAME:AddMessage("|cFFAE33FF[SoundList]|r |cFFFFFF00" .. SOUND_LIST_VERSION .. "|r loaded.")
	tinsert(UISpecialFrames, "SoundListFrame")

	SLASH_SOUNDLIST1 = "/soundlist"
	SLASH_SOUNDLIST2 = "/sl"
	SlashCmdList["SOUNDLIST"] = 
		function(arg)
			SoundListFrame:Show()
		end

--    if (not maxLines) then SoundList_CreateButtons() end
	SoundList_CreateButtons()
end

---------------------------------------------------------------------------------------------------------------

function SoundListOptions_OnLoad()
	for i=1, #SOUNDLIST_KEYWORDS do
		btn = CreateFrame("Button", "sl_op_btn" .. i, SoundListOptionsFrame, "slbt_options")
		if SOUNDLIST_KEYWORDS[i] ~= "" then
			btn:SetText(SOUNDLIST_KEYWORDS[i])
--			btn:SetFormattedText("|cffff8800%s|r", SOUNDLIST_KEYWORDS[i])
			btn:SetID(i)
			btn:SetWidth(106)
			btn:SetHeight(25)
--			btn:SetPoint("TOPLEFT", SoundListOptionsFrame, "TOPLEFT", 12, 20-(btn:GetHeight()*i))
			btn:SetPoint("TOPLEFT", SoundListOptionsFrame, "TOPLEFT", 7, -9-((btn:GetHeight()+1)*i))
			btn:SetScript("OnClick", function(self, button) SoundList_ToggleSoundsList(self:GetID()) end)
		end
	end

	SoundListOptionsFrameSearch:SetScript("OnEnterPressed", function(self, userInput) SoundList_ToggleSoundsList(SoundListOptionsFrame.selectedID) end)
	SoundListOptionsFrameSearch:SetScript("OnTextChanged", SoundListOptionsFrameSearch:GetScript("OnEnterPressed"))
--	SoundListOptionsFrameSearch:SetScript("OnTabPressed", SoundListOptionsFrameSearch:GetScript("OnEnterPressed"))
	SoundListOptionsFrameFullPath:SetScript("OnClick", function(self, button, down) PlaySound("igMainMenuOptionCheckBoxOff") SoundList_ToggleSoundsList(SoundListOptionsFrame.selectedID) end)

--	SoundListOptionsFrame:SetHeight(26 * #SOUNDLIST_KEYWORDS + 3)
--	SoundListOptionsFrame:SetHeight(26 * #SOUNDLIST_KEYWORDS + 27)
	SoundListOptionsFrame:SetHeight(SoundListFrame:GetHeight())
	SoundList_ToggleSoundsList(1)
end

---------------------------------------------------------------------------------------------------------------

function SoundListOptions_Search(keyword)
--print("- SoundListOptions_Search: " .. keyword)
	local str, str_upper
	local SOUND_LIST_SEARCH = {}
	keyword = string.upper(keyword)
--	wipe(SOUND_LIST)
	for i=1, #SOUND_LIST do
		str = SOUND_LIST[i]

		if SoundListOptionsFrameFullPath:GetChecked() then
			str_upper = string.upper(str)
		else
			str_upper = {strsplit("/", str)}
			str_upper = string.upper(str_upper[#str_upper])
		end

		if string.find(str_upper, keyword) then
			table.insert(SOUND_LIST_SEARCH, str)
		end
	end
	SOUND_LIST = SOUND_LIST_SEARCH
end

---------------------------------------------------------------------------------------------------------------

function SoundList_OnShow(self)
	PlaySound("igMainMenuOpen")
	HideUIPanel(GameMenuFrame)
--	SoundListFrameScrollBar_Update()
end

---------------------------------------------------------------------------------------------------------------

function SoundList_OnHide(self)
	PlaySound("igMainMenuOption")
	SoundList_StopSound()
end

---------------------------------------------------------------------------------------------------------------

function SoundList_ListButton_OnEnter(self, motion)
	--GameTooltip:Hide()
	GameTooltip:SetBackdropColor(0, 0, 0, 1)
	--GameTooltip:SetOwner(SoundListFrame, "ANCHOR_TOPLEFT")
	GameTooltip:SetOwner(SoundListFrame, "ANCHOR_BOTTOM")
--GameTooltip:SetText("List button operations", 1, 1, 1)
--	GameTooltip:AddLine(" ")
	GameTooltip:AddDoubleLine("Path:", SOUND_LIST[self:GetID()], 1, 0.9, 0, 1, 1, 1)
	GameTooltip:AddLine(" ")
	GameTooltip:AddDoubleLine("Left Click", "Play / Stop", 1, 0.9, 0, 1, 1, 1)
	GameTooltip:AddDoubleLine("Right Click", "Copy sound string", 1, 0.9, 0, 1, 1, 1)
	GameTooltip:AddDoubleLine("Shift+Left Click", "Print sound string to chat", 1, 0.9, 0, 1, 1, 1)
	GameTooltip:Show()
end

---------------------------------------------------------------------------------------------------------------

function SoundList_TitleButton_OnEnter(self, motion)
	--GameTooltip:Hide()
	GameTooltip:SetBackdropColor(0, 0, 0, 1)
	GameTooltip:SetOwner(SoundListFrame, "ANCHOR_BOTTOM")
--GameTooltip:SetText("Title button operations", 1, 1, 1)
	--GameTooltip:AddDoubleLine("Left Click + Drag", "Move Window", 1, 0.9, 0, 1, 1, 1)
	GameTooltip:AddDoubleLine("Left Click + drag", "Move window", 1, 0.9, 0, 1, 1, 1)
	GameTooltip:AddDoubleLine("Right Click", "Toggle Options", 1, 0.9, 0, 1, 1, 1)
	GameTooltip:AddDoubleLine("Middle Click", "Toggle Name / Full path", 1, 0.9, 0, 1, 1, 1)
	GameTooltip:AddDoubleLine("Mouse Wheel", "Scale window", 1, 0.9, 0, 1, 1, 1)
	GameTooltip:AddDoubleLine("Esc", "Close Window", 1, 0.9, 0, 1, 1, 1)
	GameTooltip:Show()
end

---------------------------------------------------------------------------------------------------------------

function SoundList_TitleButton_OnMouseDown(self, button)
	SoundListTitleButtonTexture:SetGradientAlpha("VERTICAL", 0, 0, 0, 0.5, math.random(100)/100, math.random(100)/100, math.random(100)/100, 0.5)
	--SoundListTitleButtonTexture:SetGradientAlpha("VERTICAL", 0.1, 0.1, 0.1, 0, 0.25, 0.25, 0.25, 0.8);

	if ( (button == "LeftButton") and (not self:GetParent().isMoving) ) then
		self:GetParent():StartMoving()
		self:GetParent().isMoving = true
	end
end

---------------------------------------------------------------------------------------------------------------

function SoundList_TitleButton_OnMouseUp(self, button)

	SoundListTitleButtonTexture:SetGradientAlpha("VERTICAL", 1, 0.8, 0, 0.7, 0, 0.6, 0, 0.5)

	if ( (button == "LeftButton") and (self:GetParent().isMoving) ) then
		self:GetParent():StopMovingOrSizing()
		self:GetParent().isMoving = false

	elseif (button == "RightButton") then
		SoundListOptions_ShowOptions()

	elseif (button == "MiddleButton") then
		SoundListFrame.fullPath = not SoundListFrame.fullPath
		if SoundListFrame.fullPath then
			SoundListFrame:SetWidth(814)
			SoundListFrame.title:SetWidth(802)
		else
			SoundListFrame:SetWidth(620)
			SoundListFrame.title:SetWidth(608)
		end
		SoundList_RefreshList()

	end
end

---------------------------------------------------------------------------------------------------------------

function SoundList_TitleButton_OnMouseWheel(self, delta)
--	if IsControlKeyDown() then
		SoundListFrame:SetScale(SoundListFrame:GetScale() + delta / 20)
--	end
end

---------------------------------------------------------------------------------------------------------------

function SoundListFrameScrollBar_Update()
--print("SoundListFrameScrollBar_Update()")
	local i, lineplusoffset, title
	FauxScrollFrame_Update(SoundListFrameScrollBar, maxLines, maxLinesToShow, 10)

	for i=1, maxLinesToShow do
		lineplusoffset = i + FauxScrollFrame_GetOffset(SoundListFrameScrollBar)

		local btn = _G["slb" .. i]

		if (lineplusoffset <= maxLines) then
			btn:SetID(lineplusoffset)

			if SoundListFrame.fullPath then
				btn:SetText(format("|cff00FF00%s|r  |cffFFFF00%s|r", lineplusoffset, SOUND_LIST[lineplusoffset]))
				btn:SetWidth(784)
			else
				title = {strsplit("/", SOUND_LIST[lineplusoffset])}
				btn:SetText(format("|cff00FF00%s|r  |cffFFFF00%s|r", lineplusoffset, title[#title]))
				btn:SetWidth(590)
			end

			if selectedButtonID == lineplusoffset and soundHandle ~= nil then
				btn:GetNormalTexture():SetGradientAlpha("VERTICAL", 1, 0.8, 0, 0.7, 0, 0.6, 0, 0.5)
--					btn:GetNormalTexture():SetGradientAlpha("VERTICAL", 0, 0.3, 0, 0.75, 0, 1, 0, 0.53)
			else
				btn:GetNormalTexture():SetGradientAlpha("VERTICAL", 0.1, 0.1, 0.1, 0, 0.25, 0.25, 0.25, 0.8)
			end

			btn:Show()
		else
			btn:Hide()
		end

	end
end

---------------------------------------------------------------------------------------------------------------

function SoundList_ListButton_OnMouseDown(self, button, down)
--	if down then return end
	local btnID = self:GetID()
	local sound_file = SOUND_LIST[btnID]

--msg(format("A  -  selectedButtonID = %i     btnID = %i     soundHandle = %s", selectedButtonID, btnID, tostring(soundHandle)))

	if IsShiftKeyDown() then
		--DEFAULT_CHAT_FRAME:AddMessage(sound_file)
		DEFAULT_CHAT_FRAME:AddMessage(self:GetText())
		return
	end

	if button == "LeftButton" then

--zd({ soundHandle = soundHandle, selectedButtonID = selectedButtonID, btnID = btnID })


		if soundHandle then StopSound(soundHandle) end

		if selectedButtonID ~= btnID or soundHandle == nil then
			willPlay, soundHandle = PlaySoundFile(sound_file)
			DEFAULT_CHAT_FRAME:AddMessage("Playing: " .. self:GetText())
		else
			soundHandle = nil
		end

		-- SAME RESULT !!
		--		if soundHandle then 
		--			StopSound(soundHandle)
		--			soundHandle = nil
		--			--selectedButtonID = nil
		--			if selectedButtonID ~= btnID then
		--				willPlay, soundHandle = PlaySoundFile(sound_file)
		--			end
		--		else
		--			willPlay, soundHandle = PlaySoundFile(sound_file)
		--		end

		selectedButtonID = btnID
		SoundListFrameScrollBar_Update()

	else
		local edit_box = ChatEdit_ChooseBoxForSend()
		ChatEdit_ActivateChat(edit_box)
		edit_box:Insert(format('/run PlaySoundFile("%s")', sound_file))
		edit_box:HighlightText()
	end

--msg(format("B -  selectedButtonID = %i     btnID = %i     soundHandle = %s", selectedButtonID, btnID, tostring(soundHandle)))

end









--[[


C_Timer.After(0.1, 
function()
	SoundListFrame:Show()
	--SoundListFrame:SetScale(2)
	LoadAddOn("!zDevTools")
	zDevToolsFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 800, 0)
	zDevToolsDebugFrameAddonToWatchButton:SetText("SoundList")
	zDevToolsFrame:SetChecked(true)
	zDevTools_OnToggle(zDevToolsFrame)
SoundListOptions_ShowOptions()
end)
]]
